var buster = require("buster");
var assert = buster.assert;
var terminal = require("../../lib/buster-terminal");
var labeledList = require("../../lib/buster-terminal/labeled-list");

// For debugging when comparing space padded strings: visualizes all whitespace
function s(str) {
    return str.replace(/\n/g, "\\n").replace(/ /g, "\\s");
}

buster.testCase("Buster terminal labeled list test", {
    setUp: function () {
        this.io = {
            out: "",
            print: function (str) { this.out += str; },
            puts: function (str) { this.out += str + "\n"; },
            toString: function () { return this.out; }
        };

        this.stub(terminal, "save", function () { return "S+"; });
        this.stub(terminal, "restore", function () { return "R+"; });
        this.stub(terminal, "up", function (n) { return n == 0 ? "" : "U"+n+"+"; });
        this.stub(terminal, "fwd", function (n) { return n == 0 ? "" : "F"+n+"+"; });
    },

    "should print labels in equally spaced cells": function () {
        var list = labeledList.create(this.io, "Internet Explorer", "Firefox");

        assert.equals("Internet Explorer: \nFirefox:           \n", this.io.out);
    },

    "should print dot for first item": function () {
        var list = labeledList.create(this.io, "Internet Explorer", "Firefox");
        list.print("Internet Explorer", ".");

        assert.equals(s("Internet Explorer: \nFirefox:           \n" +
                        "S+U2+F19+.R+"), s(this.io.out));
    },

    "should print three markers in alternating lines": function () {
        var list = labeledList.create(this.io, "Internet Explorer", "Firefox");
        list.print("Internet Explorer", ".");
        list.print("Firefox", ".");
        list.print("Internet Explorer", "E");

        assert.equals(s("Internet Explorer: \nFirefox:           \n" +
                        "S+U2+F19+.R+S+U1+F19+.R+S+U2+F20+ER+"), s(this.io.out));
    },

    "should add label": function () {
        var list = labeledList.create(this.io);
        list.add("Internet Explorer");
        list.add("Firefox");
        list.print("Internet Explorer", ".");

        assert.equals(s("Internet Explorer: \nFirefox:           \nS+U2+F19+.R+"),
                      s(this.io.out));
    },

    "should refit labels when adding a wider one": function () {
        var list = labeledList.create(this.io);
        list.add("Firefox");

        assert.equals("Firefox: \n", this.io.out);

        list.add("Internet Explorer");

        assert.equals("Firefox: \nS+U1+Firefox:           R+Internet Explorer: \n",
                      this.io.out);
    },

    "should save contents when adding and refitting labels": function () {
        var list = labeledList.create(this.io);
        list.add("Firefox");
        list.print("Firefox", ".");
        list.print("Firefox", ".");
        list.add("Internet Explorer");
        list.add("Chrome");
        list.print("Firefox", "F");
        list.print("Internet Explorer", ".");

        assert.equals("Firefox: \nS+U1+F9+.R+S+U1+F10+.R+S+U1+Firefox:           " +
                      "..R+Internet Explorer: \nChrome:            " +
                      "\nS+U3+F21+FR+S+U2+F19+.R+", this.io.out);
    }
});
